@@ -9,41 +9,39 @@
 #	<view> window
 #	<view> date
 #	<view> set_date <date>
-#	<view> unselect <item>
-#	<view> select <item>
-#	<view> itemwindow <item>
+
+#### A null view to be used if nothing is selected ####
+
+proc ical_null_view {method args} {
+    switch -exact -- $method {
+	window		{return .}
+	date		{return [date today]}
+	set_date	{}
+	default		{error "unknown view method: $method"}
+    }
+}
 
 #### Helper routines ####
 
 proc ical_with_view {view body} {
-    global ical_state
-
-    set old $ical_state(view)
-    set ical_state(view) $view
-
     # XXX Error handling?
     uplevel $body
-
-    set ical_state(view) $old
-}
-
-proc ical_action {view action args} {
-    global ical_state
-
-    set old $ical_state(view)
-    set ical_state(view) $view
-    eval [list ical_$action] $args
-    set ical_state(view)  $old
 }
 
 proc ical_view {} {
-    global ical_state
-    return $ical_state(view)
+    global ical_view
+
+    set f [ical_focus]
+    if ![info exists ical_view($f)] {
+	return ical_null_view
+    } else {
+	return $ical_view($f)
+    }
 }
 
 proc ical_leader {} {
     global ical_state
-    if ![string compare $ical_state(view) ""] {return ""}
+    if ![string compare [ical_view] ical_null_view] {return ""}
     return [[ical_view] window]
 }
 
@@ -60,12 +58,10 @@ proc ical_set_date {d} {
 }
 
 proc ical_with_item {v} {
-    global ical_state
-    if [string compare $ical_state(selview) $ical_state(view)] {return 0}
-    if ![string compare $ical_state(selitem) {}] {return 0}
+    if [catch {set item [ical_find_selection]}] {return 0}
 
     upvar $v i
-    set i $ical_state(selitem)
+    set i $item
     return 1
 }
 
@@ -107,51 +103,34 @@ proc ical_clipped {} {
     return $ical_state(clip)
 }
 
-#### Editing routines ####
+#### Creating an action routine ####
 
-# The following routines provide support for editing in item windows.
-# The basic support is provided by the Canvas routines in
-# "tcllib/edit.tcl".  Here we just introduce wrappers around
-# modification operations
-
-# called by generic editing code to delete a range of text.
-proc icalitem_edit_delete {c i1 i2} {
-    if ![ical_with_mod_single_item item] {return}
-
-    global Canvas_Edit ical_state
-    set iw $ical_state(seliw)
-    if [string compare $c [$iw canvas]] return
-    $Canvas_Edit(delete) $c $i1 $i2
-    $iw save
-}
-
-# called by generic editing code to insert text.
-proc icalitem_edit_insert {c t} {
-    if ![ical_with_mod_single_item item] {return}
-
-    global Canvas_Edit ical_state
-    set iw $ical_state(seliw)
-    if [string compare $c [$iw canvas]] return
-    $Canvas_Edit(insert) $c $t
-    $iw save
+proc action {name enabler title formals body} {
+    global action_title ical_action_enabler
+    set ical_action_enabler($name) $enabler
+    set action_title($name) $title
+    proc $name $formals $body
 }
 
 #### Action routines ####
 
-proc ical_newview {} {
+action ical_newview always {Create new ical window} {} {
     global ical_state
     return [DayView]
 }
 
-proc ical_save {} {
+action ical_save always {Save pending changes} {} {
     io_save [ical_leader]
 }
 
-proc ical_reread {} {
+action ical_reread always {Read changes made by other users} {} {
     io_read [ical_leader]
 }
 
-proc ical_exit {} {
+action ical_nop always {Do nothing} {} {
+}
+
+action ical_exit always {Kill ical} {} {
     if ![io_save [ical_leader]] return
 
     run-hook ical-exit
@@ -159,25 +138,27 @@ proc ical_exit {} {
     destroy .
 }
 
-proc ical_close {} {
+action ical_close always {Close ical window} {} {
+    # Never try to delete the null view
+    if ![string compare [ical_view] ical_null_view] return
+
     global ical_state
     if {[llength $ical_state(views)] < 2} {
 	# Try and save --- last view is about to be closed
 	if ![io_save [ical_leader]] return
 
 	class_kill [ical_view]
-	set ical_state(view) ""
 	run-hook ical-exit
 	cal delete
 	destroy .
     } else {
 	# Not the last view
 	class_kill [ical_view]
-	set ical_state(view) ""
     }
 }
 
-proc ical_cut_or_hide {} {
+
+action ical_cut_or_hide item {Delete selected item} {} {
     if ![ical_with_item i] return
 
     if {[$i owned] || ([$i calendar] == [cal main])} {
@@ -187,14 +168,14 @@ proc ical_cut_or_hide {} {
     }
 }
 
-proc ical_cut {} {
+action ical_cut witem {Delete selected item even if owned by another user} {} {
     if ![ical_with_mod_single_item i] return
 
     ical_clip $i
     cal remove $i
 }
 
-proc ical_hide {} {
+action ical_hide item {Hide selected item} {} {
     if ![ical_with_item i] return
 
     # Since the hide entry will go in the main calendar,
@@ -227,12 +208,12 @@ proc ical_hide {} {
     cal hide $i
 }
 
-proc ical_copy {} {
+action ical_copy item {Copy selected item to clipboard} {} {
     if ![ical_with_item i] return
     ical_clip [$i clone]
 }
 
-proc ical_paste {} {
+action ical_paste writable {Paste clipboard item} {} {
     if [cal readonly] {
 	ical_error "Permission denied"
 	return
@@ -247,36 +228,23 @@ proc ical_paste {} {
     set i [$i clone]
     $i date [ical_date]
     $i own
-    $i todo 0
 
     cal add $i
+    run-hook item-create $i
 }
 
-proc ical_import {} {
+action ical_import writable {Parse X selection as an item and add it to calendar} {} {
     if [catch {set sel [selection get]}] {return}
 
     if ![cal readonly] {
 	set i [item_parse $sel [ical_date]]
-	run-hook item-create $i
 	cal add $i
+	run-hook item-create $i
 	ical_set_date [$i first]
     }
 }
 
-proc ical_insert_selection {} {
-    if ![ical_with_mod_single_item i] return
-
-    if [catch {ical_edit self-insert [selection get]}] {
-	ical_edit yank
-    }
-}
-
-proc ical_delete_selection {} {
-    if ![ical_with_mod_single_item i] return
-    ical_edit delete-selection
-}
-
-proc ical_makeunique {} {
+action ical_makeunique witem {Create unique occurrence of selected item} {} {
     if ![ical_with_mod_item i] return
     if ![$i repeats] {
 	ical_error "Item does not repeat"
@@ -289,7 +257,7 @@ proc ical_makeunique {} {
     cal add $c [$i calendar]
 }
 
-proc ical_moveitem {calendar} {
+action ical_moveitem witem {Move item to another calendar} {calendar} {
     if [cal readonly $calendar] {
 	ical_error "$calendar: permission denied"
 	return
@@ -299,7 +267,7 @@ proc ical_moveitem {calendar} {
     cal add $i $calendar
 }
 
-proc ical_addinclude {} {
+action ical_addinclude writable {Include calendar} {} {
     if [cal readonly] {
 	ical_error "[cal main]: permission denied"
 	return
@@ -331,7 +299,7 @@ proc ical_addinclude {} {
     }
 }
 
-proc ical_removeinc {calendar} {
+action ical_removeinc writable {Remove included calendar} {calendar} {
     if [cal readonly] {
 	ical_error "[cal main]: permission denied"
 	return
@@ -349,62 +317,73 @@ proc ical_removeinc {calendar} {
 	    # Conflict!
 	    set query [yes_no_cancel [ical_leader]\
 		       "$calendar has been modified since last read. Save?"]
-	    if ($query == "cancel") {
+	    if {$query == "cancel"} {
 		return
 	    }
-	    if ($query == "no") {
+	    if {$query == "no"} {
 		set save 0
 	    }
 	}
 
 	if $save {
 	    if [catch {cal save $calendar} error] {
-		ical_error "$file\n\n$error"
+		ical_error "$calendar\n\n$error"
 		return
 	    }
 	}
     }
 
     # Remove it
-    cal exclude $calendar
+    if [catch {cal exclude $calendar} error] {
+	ical_error "$calendar\n\n$error"
+	return
+    }
 }
 
-proc ical_norepeat {} {
+action ical_norepeat witem {Make item non-repeating} {} {
     if ![ical_with_mod_item i] return
     $i date [ical_date]
 }
 
-proc ical_daily {} {
+action ical_daily witem {Make item repeat daily} {} {
     if ![ical_with_mod_item i] return
-    $i dayrepeat 1 [ical_date]
+    set d [ical_date]
+    $i dayrepeat 1 $d
+    $i start $d
 }
 
-proc ical_monthly {} {
+action ical_monthly witem {Make item repeat monthly} {} {
     if ![ical_with_mod_item i] return
-    $i month_day [date monthday [ical_date]]
+    set d [ical_date]
+    $i month_day [date monthday $d]
+    $i start $d
 }
 
-proc ical_annual {} {
+action ical_annual witem {Make item repeat yearly} {} {
     if ![ical_with_mod_item i] return
-    $i month_day [date monthday [ical_date]] [ical_date] 12
+    set d [ical_date]
+    $i month_day [date monthday $d] $d 12
+    $i start $d
 }
 
-proc ical_weekly {} {
+action ical_weekly witem {Make item repeat weekly} {} {
     if ![ical_with_mod_item i] return
-    $i weekdays [date weekday [ical_date]]
+    set d [ical_date]
+    $i weekdays [date weekday $d]
+    $i start $d
 }
 
-proc ical_edit_monthly {} {
+action ical_edit_monthly witem {Make item repeat monthly in a complicated way} {} {
     if ![ical_with_mod_item i] return
     monthrepeat [ical_leader] $i [ical_date]
 }
 
-proc ical_edit_weekly {} {
+action ical_edit_weekly witem {Make item repeat weekly in a complicated way} {} {
     if ![ical_with_mod_item i] return
     weekrepeat [ical_leader] $i [ical_date]
 }
 
-proc ical_set_range {} {
+action ical_set_range witem {Restrict item repetition range} {} {
     if ![ical_with_mod_item i] return
 
     if ![$i repeats] {
@@ -412,15 +391,27 @@ proc ical_set_range {} {
 	return
     }
 
-    set start [ical_date]
-    set finish [ical_date]
+    if ![$i range start finish] {
+	ical_error "Item does not repeat"
+    }
+
     if ![get_daterange [ical_leader] start finish] return
 
     $i start $start
     $i finish $finish
 }
 
-proc ical_alarms {} {
+action ical_last_date witem {Stop item repetition at this date} {} {
+    if ![ical_with_mod_item i] return
+
+    if ![$i repeats] {
+	ical_error "Item does not repeat"
+	return
+    }
+    $i finish [ical_date]
+}
+
+action ical_alarms wappt {Change appointment alarms} {} {
     if ![ical_with_mod_item i] return
 
     if [catch {set alarms [$i alarms]}] {set alarms [cal option DefaultAlarms]}
@@ -432,12 +423,88 @@ proc ical_alarms {} {
     catch {$i alarms $alarms}
 }
 
-proc ical_remind {n} {
+action ical_edit_item witem {Edit item properties} {} {
+    if ![ical_with_mod_item i] return
+    item_edit [ical_leader] $i
+}
+
+action ical_link_to_file witem {Create a link to a file} {} {
+    if ![ical_with_mod_item i] return
+
+    # Try to get good initial file name
+    set initial {}
+    if ![catch {set link [$i option Link]}] {
+	if [regexp {^file://localhost/(.*)$} $link junk filename] {
+	    set initial /$filename
+	}
+    }
+
+    if ![get_file_name [ical_leader] "Item link"\
+	     "Select file to which link should be created."\
+	     filename $initial] return
+
+    $i option Link file://localhost$filename
+}
+
+action ical_link_to_uri witem {Create a link to a Web document} {} {
+    if ![ical_with_mod_item i] return
+
+    set initial {}
+    catch {set initial [$i option Link]}
+
+    if ![get_string [ical_leader] "Document Locator"\
+	     "Enter the uniform resource identifier of document"\
+	     $initial result] return
+    $i option Link $result
+}
+
+action ical_remove_link witem {Remove any link from item} {} {
+    if ![ical_with_mod_item i] return
+
+    if [catch {$i delete_option Link} msg] {
+	ical_error {Item does not contain a link}
+    }
+}
+
+action ical_follow_link item {Follow link from item} {} {
+    if ![ical_with_item i] return
+
+    if [catch {set uri [$i option Link]}] {
+	ical_error "Item does not have a link."
+	return
+    }
+
+    follow_link $uri
+}
+
+action ical_deflistings writable {Set default value for item early warnings} {} {
+    set num [cal option DefaultEarlyWarning]
+    if [get_number [ical_leader] {Early warning}\
+	    {Days}\
+	    {By default items show up in item listings this many days early}\
+	    0 15 5 $num num] {
+	cal option DefaultEarlyWarning $num
+    }
+}
+
+action ical_remind witem {Set early warning option for item} {n} {
     if ![ical_with_mod_item i] return
     $i earlywarning $n
 }
 
-proc ical_set_owner {} {
+action ical_set_remind witem {Set early warning option for item} {} {
+    if ![ical_with_mod_item i] return
+
+    set num [$i earlywarning]
+    if [get_number [ical_leader] {Early warning}\
+	    {Days}\
+	    {By default this item shows up in item listings this many days early}\
+	    0 15 5 $num num] {
+	$i earlywarning $num
+    }
+}
+
+action ical_set_owner witem {Change item owner} {} {
     if ![ical_with_mod_single_item i] return
 
     if ![get_string [ical_leader] "Owner" "Enter owner name"\
@@ -445,28 +512,58 @@ proc ical_set_owner {} {
     $i owner $result
 }
 
-proc ical_hilite {mode} {
+action ical_hilite witem {Set highlight mode for item} {mode} {
     if ![ical_with_mod_single_item i] return
     $i hilite $mode
 }
 
-proc ical_toggle_todo {} {
+action ical_toggle_todo witem {Make item a todo item} {} {
     if ![ical_with_mod_single_item i] return
     $i todo [expr ![$i todo]]
 }
 
-proc ical_print {} {
+action ical_toggle_done witem {Mark todo item as done} {} {
+    if ![ical_with_mod_item i] return
+    if ![$i todo] {
+	ical_error "Item is not a todo item"
+	return
+    }
+
+    if [$i is_done] {
+	$i done 0
+	return
+    }
+
+    set date [ical_date]
+
+    # Make the current instance of the item unique
+    if [$i repeats] {
+	# XXX Make clone that does not occur on or before the current date
+	set c [$i clone]
+	$c start [expr $date+1]
+	cal add $c [$i calendar]
+    }
+
+    # Modify this occurrence
+    $i date $date
+    $i done 1
+    $i hilite never
+
+    run-hook todo-item-done $i
+}
+
+action ical_print always {Print calendar contents} {} {
     if [catch {print_calendar [ical_leader] [ical_date]} msg] {
 	ical_error $msg
     }
 }
 
-proc ical_viewitems {calendar} {
+action ical_viewitems always {View all items from a calendar} {calendar} {
     set l [ItemListing]
     $l calendar $calendar
 }
 
-proc ical_list {n} {
+action ical_list always {List items from selected range of dates} {n} {
     set start [ical_date]
     if {$n == "week"} {
 	set start [expr $start+1-[date weekday $start]]
@@ -491,26 +588,26 @@ proc ical_list {n} {
     $l dayrange $start [expr $start+$n-1]
 }
 
-proc ical_toggle_overflow {} {
+action ical_toggle_overflow writable {Allow text to overflow appointment boundaries?} {} {
     if [cal readonly] {return}
     cal option AllowOverflow [expr ![cal option AllowOverflow]]
 }
 
-proc ical_toggle_ampm {} {
+action ical_toggle_ampm writable {Display time with am/pm indicators?} {} {
     if [cal readonly] {return}
 
     cal option AmPm [expr ![cal option AmPm]]
     trigger fire reconfig
 }
 
-proc ical_toggle_monday {} {
+action ical_toggle_monday writable {Display Monday at the start of a week?} {} {
     if [cal readonly] {return}
 
     cal option MondayFirst [expr ![cal option MondayFirst]]
     trigger fire reconfig
 }
 
-proc ical_timerange {} {
+action ical_timerange writable {Set the range of time initially displayed in a window} {} {
     if [cal readonly] {return}
 
     set start [cal option DayviewTimeStart]
@@ -528,34 +625,33 @@ proc ical_timerange {} {
     }
 }
 
-proc ical_noticeheight {} {
+action ical_noticeheight writable {Change the default height of the notice window} {} {
     if [cal readonly] {return}
 
     set ht [cal option NoticeHeight]
-    if [get_number [ical_leader] {Notice Window Height} {In centimeters}\
+    if [get_number [ical_leader] {Notice Window Height}\
+	    {Centimeters}\
+	    {Specify the height of the notice window}\
 	    1 15 0 $ht ht] {
 	cal option NoticeHeight $ht
 	trigger fire reconfig
     }
 }
 
-proc ical_itemwidth {} {
+action ical_itemwidth writable {Change the default width of the notice and appt windows} {} {
     if [cal readonly] {return}
 
     set w [cal option ItemWidth]
-    if [get_number [ical_leader] {Item Width} {In centimeters}\
+    if [get_number [ical_leader] {Item Width}\
+	    {Centimeters}\
+	    {Specify the width of appointments and notices}\
 	    5 15 0 $w w] {
 	cal option ItemWidth $w
 	trigger fire reconfig
     }
 }
 
-proc ical_defremind {n} {
-    if [cal readonly] {return}
-    cal option DefaultEarlyWarning $n
-}
-
-proc ical_defalarms {} {
+action ical_defalarms writable {Change default alarm settings} {} {
     if [cal readonly] {return}
 
     set alarms [cal option DefaultAlarms]
@@ -568,27 +664,52 @@ proc ical_defalarms {} {
     alarmer recompute
 }
 
-proc ical_gripe {} {
+action ical_cmdkey always {Change command key bindings} {} {
+    global keymap
+
+    if [edit_key_binding keymap(command)] {
+	# Redo the binding map and set-up menu accelerators
+	ical_make_bindings IcalCommand $keymap(command)
+	trigger fire keybind
+    }
+}
+
+proc edit_key_binding {var} {
+    upvar $var map
+    if ![define_key [ical_leader] key] {return 0}
+
+    # Remove duplicates from map
+    lappend map $key
+    foreach x $map {set val([lindex $x 0]) [lindex $x 1]}
+
+    # Create new map
+    set map {}
+    foreach key [lsort [array names val]] {
+	if [string compare $val($key) ""] {
+	    lappend map [list $key $val($key)]
+	}
+    }
+
+    return 1
+}
+
+action ical_gripe always {Gripe to the author of ical} {} {
     global ical
     bug_notify $ical(mailer) $ical(author) {Gripe}
 }
 
-proc ical_edit {args} {
-    global ical_state
-    set fw $ical_state(seliw)
-    if ![string compare $fw ""] {return}
-
-    eval edit_action IcalItem_Edit [$fw canvas] $args
-}
-
-proc ical_help {} {
+action ical_help always {Display help on ical} {} {
     global ical
     Ical_Doc $ical(doc)
 }
 
+action ical_about always {Display information about this version of ical} {} {
+    show_about [ical_leader]
+}
+
 #### Date change routines ####
 
-proc ical_last_month {} {
+action ical_last_month always {Move to previous month} {} {
     set split [date split [ical_date]]
     set month [lindex $split 2]
     set year [lindex $split 3]
@@ -609,28 +730,7 @@ proc ical_last_month {} {
     ical_set_date [date make $day $month $year]
 }
 
-proc ical_last_month {} {
-    set split [date split [ical_date]]
-    set month [lindex $split 2]
-    set year [lindex $split 3]
-    if {$month == 1} {
-	set month 12
-	incr year -1
-    } else {
-	incr month -1
-    }
-
-    # Handle range errors
-    if [catch {set first [date make 1 $month $year]}] {return}
-
-    # Adjust monthday to month size
-    set day [lindex $split 0]
-    if {$day > [date monthsize $first]} {set day [date monthsize $first]}
-
-    ical_set_date [date make $day $month $year]
-}
-
-proc ical_next_month {} {
+action ical_next_month always {Move to next month} {} {
     set split [date split [ical_date]]
     set month [lindex $split 2]
     set year [lindex $split 3]
@@ -651,7 +751,7 @@ proc ical_next_month {} {
     ical_set_date [date make $day $month $year]
 }
 
-proc ical_last_year {} {
+action ical_last_year always {Move to previous year} {} {
     set split [date split [ical_date]]
     set month [lindex $split 2]
     set year [lindex $split 3]
@@ -667,7 +767,7 @@ proc ical_last_year {} {
     ical_set_date [date make $day $month $year]
 }
 
-proc ical_next_year {} {
+action ical_next_year always {Move to next year} {} {
     set split [date split [ical_date]]
     set month [lindex $split 2]
     set year [lindex $split 3]
@@ -683,41 +783,150 @@ proc ical_next_year {} {
     ical_set_date [date make $day $month $year]
 }
 
-proc ical_today {} {
+action ical_today always {Move to today} {} {
     ical_set_date [date today]
 }
 
-proc ical_last_day {} {
+action ical_last_day always {Move to previous day} {} {
     set date [expr [ical_date]-1]
     if {$date < [date first]} {return}
     ical_set_date $date
 }
 
-proc ical_next_day {} {
+action ical_next_day always {Move to next day} {} {
     set date [expr [ical_date]+1]
     if {$date > [date last]} {return}
     ical_set_date $date
 }
 
-#### Item selection routines ####
-
-proc ical_select {item} {
-    ical_unselect
-
-    global ical_state
-    set view [ical_view]
-    set ical_state(selview) $view
-    set ical_state(selitem) $item
-    set ical_state(seliw)   [$view itemwindow $item]
-    $view select $item
+action ical_last_week always {Move to last week} {} {
+    set date [expr [ical_date]-7]
+    if {$date < [date first]} {return}
+    ical_set_date $date
 }
 
-proc ical_unselect {} {
-    global ical_state
-    if [string compare $ical_state(selitem) {}] {
-	$ical_state(selview) unselect $ical_state(selitem)
+action ical_next_week always {Move to next week} {} {
+    set date [expr [ical_date]+7]
+    if {$date > [date last]} {return}
+    ical_set_date $date
+}
+
+action ical_cycle_through_items always {Select next item} {} {
+    set found_it 0
+    if ![ical_with_item cur] {
+	set cur {}
+	set found_it 1
     }
-    set ical_state(selview) {}
-    set ical_state(selitem) {}
-    set ical_state(seliw)   {}
+
+    cal query [ical_date] [ical_date] i d {
+	if $found_it {
+	    ical_select $i $d
+	    return
+	}
+	if ![string compare $i $cur] {set found_it 1}
+    }
+
+    ical_unselect
+}
+
+action ical_search_forward always {Search forward} {} {
+    global ical_state
+    set current [ical_date]
+
+    set opat $ical_state(search)
+    if ![get_string [ical_leader] "Search" "Enter search string" $opat pat] {
+	return
+    }
+    set ical_state(search) $pat
+
+    # Convert pattern to regular expression
+    regsub -all {[^a-zA-Z0-9_]} $pat {\\&} pat
+
+    if [ical_with_item cur] {
+	# Special handling for current day
+	set list {}
+	cal query $current $current i d {
+	    lappend list $i
+	}
+
+	# Drop all items upto and including "cur"
+	foreach i [lrange $list [expr [lsearch -exact $list $cur]+1] end] {
+	    if [regexp -nocase -- $pat [$i text]] {
+		ical_select $i $current
+		return
+	    }
+	}
+	incr current
+    }
+	    
+    # Get list of items we want to query over
+    set list {}
+    cal loop i {
+	if [regexp -nocase -- $pat [$i text]] {
+	    lappend list $i
+	}
+    }
+
+    # Now search for next occurrence
+    cal loop_forward -items $list {} $current i d {
+	ical_set_date $d
+	ical_select $i $d
+	return
+    }
+
+    error_notify [ical_leader] "No more items"
+}
+
+action ical_search_backward always {Search backward} {} {
+    global ical_state
+    set current [ical_date]
+
+    set opat $ical_state(search)
+    if ![get_string [ical_leader] "Search" "Enter search string" $opat pat] {
+	return
+    }
+    set ical_state(search) $pat
+
+    # Convert pattern to regular expression
+    regsub -all {[^a-zA-Z0-9_]} $pat {\\&} pat
+
+    if [ical_with_item cur] {
+	# Special handling for current day
+	set list {}
+	cal query $current $current i d {
+	    lappend list $i
+	}
+
+	# Drop all items from "cur" onwards
+	set last {}
+	foreach i [lrange $list 0 [expr [lsearch -exact $list $cur]-1]] {
+	    if [regexp -nocase -- $pat [$i text]] {
+		set last $i
+	    }
+	}
+
+	if [string compare $last {}] {
+	    ical_select $last $current
+	    return
+	}
+
+	incr current -1
+    }
+
+    # Get list of items we want to query over
+    set list {}
+    cal loop i {
+	if [regexp -nocase -- $pat [$i text]] {
+	    lappend list $i
+	}
+    }
+
+    # Now search for previous occurrence
+    cal loop_backward -items $list {} $current i d {
+	ical_set_date $d
+	ical_select $i $d
+	return
+    }
+
+    error_notify [ical_leader] "No more items"
 }
