@@ -151,10 +151,7 @@ class AlarmNotice {item starttime} {
     set slot(item) $item
     set slot(starttime) $starttime
 
-    toplevel .$self -class Reminder -bd 0 -geometry ""
-
-    focus_interest .$self
-    bind .$self <Any-Enter> [list focus_restrict .$self]
+    toplevel .$self -class Reminder
 
     wm title .$self Reminder
     wm iconname .$self Reminder
@@ -181,11 +178,11 @@ class AlarmNotice {item starttime} {
     set fi [time2text [expr [$item starttime]+[$item length]]]
 
 
-    frame .$self.top -relief raised -bd 1
-    label .$self.head -relief flat -bd 1 -text "Appointment from $st to $fi"
-    label .$self.foot -relief flat -bd 1 -text ""
+    frame .$self.top -class Pane
+    label .$self.head -text "Appointment from $st to $fi"
+    label .$self.foot -text ""
 
-    text .$self.text -relief flat -bd 1 -width 50 -height $lines -wrap word
+    text .$self.text -width 50 -height $lines -wrap word -relief groove
     .$self.text insert insert $str
     .$self.text configure -state disabled
 
@@ -208,7 +205,7 @@ class AlarmNotice {item starttime} {
     trigger on flush		[list AN_check_kill $self]
     trigger on update_alarms	[list $self countdown]
 
-    wbeep 0
+    bell
 }
 
 method AlarmNotice destructor {} {
@@ -218,8 +215,6 @@ method AlarmNotice destructor {} {
     trigger remove flush		[list AN_check_kill $self]
     trigger remove update_alarms	[list $self countdown]
 
-    focus_disinterest .$self
-    focus_unrestrict .$self
     destroy .$self
 }
 
@@ -255,13 +250,13 @@ method AlarmNotice item {} {
 method AlarmNotice countdown {} {
     set now [time now]
     if {$now > $slot(starttime)} {
-	.$self.foot configure -text "Late for Appointment"
+	catch {.$self.foot configure -text "Late for Appointment"}
 
 	# No more need for countdown triggers
-	trigger remove update_alarms [list $self countdown]
+	catch {trigger remove update_alarms [list $self countdown]}
     } else {
 	set min [format "%.f" [expr ($slot(starttime)-$now)/60]]
-	.$self.foot configure -text "in $min minutes"
+	catch {.$self.foot configure -text "in $min minutes"}
     }
 }
 
