@@ -13,15 +13,14 @@ class ApptList {name view} {
     set slot(view) $view
     set slot(date) [date today]
     set slot(items) ""
-    set slot(focus) ""
     set slot(width) 100
     set slot(start) 0
     set slot(finish) 24
+    set slot(sel) {}
 
-    frame $name -bd 0
-    scrollbar $name.s -relief raised -bd 1 -orient vertical\
-	-command [list $name.c yview]
-    canvas $name.c -bd 1 -relief raised -yscroll [list $name.s set]
+    frame $name -class ApptList
+    scrollbar $name.s -orient vertical -command [list $name.c yview]
+    canvas $name.c -yscrollcommand [list $name.s set]
 
     # Get font for this window
     set slot(font) [option get $name.c itemFont Font]
@@ -39,10 +38,11 @@ class ApptList {name view} {
     pack $name.c -side left -expand 1 -fill both
 
     # Establish bindings
-    $name.c bind all <2> [list $name.c scan mark 0 %y]
-    $name.c bind all <B2-Motion> [list $name.c scan dragto 0 %y]
+    $name.c bind rest <2> [list $name.c scan mark 0 %y]
+    $name.c bind rest <B2-Motion> [list $name.c scan dragto 0 %y]
     $name.c bind rest <Button-1> [list $self new %y]
     bind $name.c <Configure> [list $self canvas_resize %w %h]
+    bindtags $name.c [list $name.c IcalItemEdit]
 
     # Handle triggers
     trigger on add	[list $self change]
@@ -51,6 +51,7 @@ class ApptList {name view} {
     trigger on text	[list $self textchange]
     trigger on flush	[list $self rescan]
     trigger on reconfig	[list $self reconfig]
+    trigger on select	[list $self check_selection]
 }
 
 method ApptList set_date {date} {
@@ -71,13 +72,14 @@ method ApptList destructor {} {
     trigger remove text		[list $self textchange]
     trigger remove flush	[list $self rescan]
     trigger remove reconfig	[list $self reconfig]
+    trigger remove select	[list $self check_selection]
 
-    # Trim item list to avoid unselect callback problems
+    # Trim item list
     set list $slot(items)
     set slot(items) {}
 
     foreach item $list {
-	class_kill $slot(window.$item)
+	catch {class_kill $slot(window.$item)}
     }
 
     destroy $slot(window)
@@ -87,29 +89,6 @@ method ApptList line_height {} {
     return $slot(font_height)
 }
 
-method ApptList select {item} {
-    if [info exists slot(window.$item)] {
-	set slot(focus) $item
-	$slot(window.$item) select
-	$self layout
-    }
-}
-
-method ApptList unselect {item} {
-    if ![string compare $slot(focus) $item] {
-	set slot(focus) {}
-	$slot(window.$item) unselect
-	$self layout
-    }
-}
-
-method ApptList itemwindow {item} {
-    if [info exists slot(window.$item)] {
-	return $slot(window.$item)
-    }
-    return {}
-}
-
 ##############################################################################
 # Internal Procedures
 
@@ -139,10 +118,13 @@ method ApptList background {} {
 	-width $width\
 	-height [expr $lines * $slot(font_height)]\
 	-confine 1\
-	-scrollincrement $slot(font_height)\
 	-scrollregion [list 0 0 $width $height]		    
-    $c xview 0
-    $c yview [expr $slot(start)*2]
+
+    # Set scrolling increment and initial position
+    $c configure -xscrollincrement $slot(font_height)
+    $c configure -yscrollincrement $slot(font_height)
+    $c xview moveto 0
+    $c yview moveto [expr $slot(start)/24]
 
     # Create background
     $c create rectangle 0 0 $width $height\
@@ -158,7 +140,7 @@ method ApptList background {} {
 
     set time 0
     for {set i 0} {$i < 48} {incr i} {
-	set ypos [expr $i * $slot(font_height)]
+	set ypos [expr $i * $slot(font_height) - 1]
 
 	if {($i % 2) != 0} {
 	    set stipple gray50
@@ -187,9 +169,9 @@ method ApptList background {} {
 }
 
 method ApptList new {y} {
-    if {$slot(focus) != ""} {
-	# Just unselect
-	ical_action $slot(view) unselect
+    # Check if something already selected on this view
+    if ![catch {set i [ical_find_selection]}] {
+	ical_unselect
 	return
     }
 
@@ -210,36 +192,28 @@ method ApptList new {y} {
     ical_with_view $slot(view) {run-hook item-create $id}
 
     if [info exists slot(window.$id)] {
-	ical_action $slot(view) select $id
+	ical_select $id $slot(date)
     }
 }
 
 method ApptList change {item} {
-    set list $slot(items)
     if {[$item is appt] && [$item contains $slot(date)]} {
-	if {[lsearch $list $item] < 0} {
-	    # Add item
-	    lappend list $item
-	    set slot(items) $list
-	    $self make_window $item
-	} else {
+	if [info exists slot(window.$item)] {
 	    $slot(window.$item) read
+	} else {
+	    # Add item
+	    lappend slot(items) $item
+	    $self make_window $item
 	}
 	$self layout
 	return
     }
 
-    if [lremove list $item] {
-	set slot(items) $list
-	class_kill $slot(window.$item)
-	unset slot(window.$item)
-	unset slot(adjust.$item)
-	$self layout
-    }
+    $self remove $item
 }
 
 method ApptList textchange {item} {
-    if {[$item is appt] && [$item contains $slot(date)]} {
+    if [info exists slot(window.$item)] {
 	$slot(window.$item) read
     }
 }
@@ -248,22 +222,26 @@ method ApptList remove {item} {
     set list $slot(items)
     if [lremove list $item] {
 	set slot(items) $list
-	class_kill $slot(window.$item)
-	unset slot(window.$item)
-	unset slot(adjust.$item)
+	$self kill $item
 	$self layout
     }
 }
 
+method ApptList kill {item} {
+    if ![info exists slot(window.$item)] return
+
+    catch {class_kill $slot(window.$item)}
+    catch {unset slot(window.$item)}
+    catch {unset slot(adjust.$item)}
+}
+
 # args are ignored - they just allow trigger to call us directly.
 method ApptList rescan {args} {
     set list $slot(items)
     set slot(items) ""
 
     foreach appt $list {
-	class_kill $slot(window.$appt)
-	unset slot(window.$appt)
-	unset slot(adjust.$appt)
+	$self kill $appt
     }
 
     set list {}
@@ -302,25 +280,40 @@ method ApptList scroll_default {} {
 	set start $minLine
     }
 
-    $slot(window).c yview $start
+    $slot(window).c yview moveto [expr double($start)/48]
 }
 
 method ApptList time {y} {
-    return [expr "($y * 30) / $slot(font_height)"]
+    return [expr int((($y + 1) * 30) / $slot(font_height))]
 }
 
 method ApptList coordinate {time} {
-    return [expr "($time * $slot(font_height)) / 30"]
+    return [expr "($time * $slot(font_height)) / 30 - 1"]
+}
+
+method ApptList check_selection {args} {
+    # Get newly selected item if it belongs to this window
+    set newsel {}
+    if ![string compare [ical_focus] [winfo toplevel $slot(window)]] {
+	# This window is active, try to get the selected item
+	catch {set newsel [ical_find_selection]}
+    }
+
+    if [string compare $newsel $slot(sel)] {
+	# Selection has changed
+	set slot(sel) $newsel
+	$self layout
+    }
 }
 
 method ApptList layout {} {
     $self sortitems
 
     # Move current appt to end of list so it appears at top
-    if {$slot(focus) != ""} {
+    if {$slot(sel) != ""} {
 	set list $slot(items)
-	if [lremove list $slot(focus)] {
-	    lappend list $slot(focus)
+	if [lremove list $slot(sel)] {
+	    lappend list $slot(sel)
 	}
 	set slot(items) $list
     }
@@ -379,7 +372,6 @@ method ApptList sortitems {} {
 method ApptList make_window {item} {
     set slot(adjust.$item) 0
     set slot(window.$item) [ApptItemWindow\
-				$slot(view)\
 				$slot(window).c\
 				$slot(font)\
 				$item $slot(date)\
@@ -415,6 +407,7 @@ method ApptList canvas_resize {w h} {
 method ApptList move {item y} {
     if {$y == "done"} {
 	$item starttime $slot(itemstart)
+	unset slot(itemstart)
 	return
     }
 
@@ -428,8 +421,15 @@ method ApptList move {item y} {
 
 method ApptList resize {item top bot} {
     if {$top == "done"} {
-	$item starttime $slot(itemstart)
-	$item length $slot(itemlength)
+	# slot(itemstart) or slot(itemlength) may not have been set yet.
+	if {[info exists slot(itemstart)] && [info exists slot(itemlength)]} {
+	    $item starttime $slot(itemstart)
+	    $item length $slot(itemlength)
+	}
+
+	catch {unset slot(itemstart)}
+	catch {unset slot(itemlength)}
+
 	return
     }
 
