@@ -2,10 +2,10 @@
 #ifndef _DATESET_H
 #define _DATESET_H
 
-#include <stdio.h>
 #include "basic.h"
 #include "Array.h"
 #include "Date.h"
+#include "arrays.h"
 #include "smallintset.h"
 
 declareArray(DateList,Date)
@@ -53,6 +53,9 @@ class DateSet {
      */
     int empty() const;
 
+    int get_range(Date&, Date&) const;
+    // Return date clipping range
+
     /*
      * modifies	result
      * effects	If *this is not empty, sets result to the smallest element in
@@ -242,9 +245,9 @@ class DateSet {
     int read(Lexer* lexer);
 
     /*
-     * effects	Writes a string representation of *this to file.
+     * effects	Appendsa string representation of *this to buffer.
      */
-    void write(FILE* file) const;
+    void write(charArray* buffer) const;
   private:
     DateSetRep*	rep;		/* Type specific representation */
     int		normalized;	/* Rep has been normalized? */
