@@ -14,6 +14,7 @@
 
 declareOpenHashMap(OptionMapRep,char const*,char const*,hash_string,cmp_string)
 
+class charArray;
 typedef OptionMapRep_Bindings OptionMap_Bindings;
 
 class OptionMap {
@@ -24,8 +25,10 @@ class OptionMap {
     bool contains(char const* key) const;
     bool fetch(char const* key, char const*& value) const;
     void store(char const* key, char const* value);
+    void remove(char const* key);
 
     OptionMap_Bindings bindings() const;
+    void write(charArray*) const;
   private:
     OptionMapRep* rep;
 };
@@ -42,6 +45,10 @@ inline bool OptionMap::fetch(char const* key, char const*& value) const {
     return (rep->fetch(key, value));
 }
 
+inline void OptionMap::remove(char const* key) {
+    rep->remove(key);
+}
+
 inline OptionMap_Bindings OptionMap::bindings() const {
     OptionMap_Bindings b = rep;
     return b;
