@@ -10,6 +10,8 @@
 #include "Date.h"
 
 class Item_Tcl;
+class Calendar_Tcl;
+class Calendar;
 
 /*
  * Item listing sorted by occurrence date.
@@ -37,16 +39,25 @@ struct Occurrence {
 declareArray(ItemList,Item_Tcl*)
 declareArray(Occurrences,Occurrence)
 
-/* Fill list with all items */
-extern void collect_items(ItemList& list);
+extern void collect_all(Calendar_Tcl*, ItemList& list);
+// modifies	list
+// effects	Collect together all of the items contained in
+//		all of the calendar files and append them to "list".
+
+extern void collect_calendar(Calendar_Tcl*, Calendar* cal, ItemList& list);
+// modifies	list
+// effects	Append all of the items contained in "cal" to "list".
 
 /*
- * Fill list with occurrences in specified range.
+ * Fill list with any occurrences in specified range of the items
+ * mentioned in "items".
  *
  * If earlywarning is true, also consider items whose early warning
  * period falls in the specified range.
  */
-extern void collect_occurrences(Occurrences& list,
+extern void collect_occurrences(Calendar_Tcl* tcl,
+				ItemList const& items,
+				Occurrences& list,
 				Date start,
 				Date finish,
 				int  earlywarning);
@@ -56,4 +67,9 @@ extern void collect_occurrences(Occurrences& list,
  */
 extern void sort_occurrences(Occurrences&);
 
+/*
+ * Reverse occurrences.
+ */
+extern void reverse(Occurrences&);
+
 #endif /* _COLLECT_H */
