@@ -14,6 +14,17 @@ proc date2text {date} {
 	    [lindex $split 3]]
 }
 
+# effects - Return shorter unparsing of date.
+proc date2text_no_weekday {date} {
+    global month_name
+
+    set split [date split $date]
+    return [format "%s %d, %d"\
+	    $month_name([lindex $split 2])\
+	    [lindex $split 0]\
+	    [lindex $split 3]]
+}
+
 # effects - Return unparsing of time.
 #	    Time is number of minutes since midnight.
 proc time2text {time} {
