@@ -24,6 +24,7 @@ proc psmonth {date} {
     lappend output {0.3 inch        SetColumnHeight}
     lappend output {0.1 inch        SetColumnSep}
     lappend output {0.25            SetTitleFraction}
+    lappend output {10              SetFontHeight}
     lappend output "($month_name($m), $y) SetHeaderLeft"
 
     ps_printtime output
@@ -178,12 +179,13 @@ proc ps_printday {o date} {
 	lappend output "\["
 
 	# Print all the lines
-	set text [item2text $item "" "" 27]
+	set text [item2text $item "" "" 2000]
 	regsub -- "\n\$" $text "" text
 	regsub -all -- {[()\\]} $text {\\&} text
 	set lines [split $text "\n"]
 
-	lappend output [format {(%s)} [join $lines ")\n("]]
+	# Put extra space at end of all strings to help out the line breaker.
+	lappend output [format {(%s )} [join $lines " )\n("]]
 
 	lappend output "\]"
     }
