@@ -99,6 +99,15 @@ proc method {class selector arglist body} {
     } $body]
 }
 
+# effects - Rename method from "old" to "new"
+proc rename_method {class old new} {
+    upvar #0 [set class]_ops ops
+    set ops($new) $ops($old)
+    unset ops($old)
+
+    rename $class::$old $class::$new
+}
+
 # effects - Invoke selected method in superclass context
 proc super {selector args} {
     global superclass
