@@ -26,7 +26,7 @@ proc yn_make {} {
     wm title $f Confirm
     wm protocol $f WM_DELETE_WINDOW {set yn_done 0}
 
-    frame $f.top -relief raised -bd 1
+    frame $f.top -class Pane
 
     message $f.text -aspect 400 -text {Uninitialized message}
     make_buttons $f.bot 1 {
@@ -56,7 +56,7 @@ proc yn_interact {leader message y n} {
 
     # Run dialog
     set yn_done -1
-    dialog_run $leader $f $f yn_done
+    dialog_run $leader $f yn_done
 
     return $yn_done
 }
