@@ -1,4 +1,4 @@
-/* Copyright (c) 1993 by Sanjay Ghemawat */
+/* Copyright (c) 1995 by Sanjay Ghemawat */
 #ifndef _ARRAYH
 #define _ARRAYH
 
@@ -20,8 +20,9 @@
  * Macro for generating array class.
  */
 
+#define ArrayNoAssert(x) ((void)0)
 #define declareArray(T,E) declareArrayType(T,E,assert)
-#define declareUncheckedArray(T,E) declareArrayType(T,E,)
+#define declareUncheckedArray(T,E) declareArrayType(T,E,ArrayNoAssert)
 
 #define declareArrayType(ArrayType,Element,ASSERT)			      \
 class ArrayType {							      \
@@ -91,6 +92,7 @@ inline void ArrayType::size_check(int val) const {			      \
 inline ArrayType::ArrayType() {						      \
     alloc_ = 0;								      \
     size_  = 0;								      \
+    store_ = 0;								      \
 }									      \
 									      \
 inline int ArrayType::size() const {					      \
@@ -146,6 +148,7 @@ ArrayType::ArrayType(int predict) {					      \
     range_check(predict);						      \
     alloc_ = 0;								      \
     size_ = 0;								      \
+    store_ = 0;								      \
     enlarge_to(predict);						      \
     size_ = 0;								      \
 }									      \
@@ -158,6 +161,7 @@ ArrayType::ArrayType(Element const* src, int s) {			      \
     range_check(s);							      \
     alloc_ = 0;								      \
     size_  = 0;								      \
+    store_ = 0;								      \
     enlarge_to(s);							      \
     for (int i = 0; i < s; i++)						      \
 	store_[i] = src[i];						      \
@@ -166,6 +170,7 @@ ArrayType::ArrayType(Element const* src, int s) {			      \
 ArrayType::ArrayType(ArrayType const& d) {				      \
     alloc_ = 0;								      \
     size_  = 0;								      \
+    store_ = 0;								      \
     enlarge_to(d.size_);						      \
     for (int i = 0; i < size_; i++)					      \
 	store_[i] = d.store_[i];					      \
@@ -175,6 +180,7 @@ ArrayType::ArrayType(Element element, int numberOfCopies) {		      \
     range_check(numberOfCopies);					      \
     alloc_ = 0;								      \
     size_ = 0;								      \
+    store_ = 0;								      \
     enlarge_to(numberOfCopies);						      \
     for (int i = 0; i < size_; i++)					      \
 	store_[i] = element;						      \
